#!/usr/bin/python -O
#
# Examples for the xmlrpc library
#
#


import sys
sys.path.append('../')
import xmlrpc
import traceback
import string


PORT		= 8024
TIMEOUT		= 1.0
LOGLEVEL	= 0		# this is the default log level


def main():
	global TESTMAP

	TESTMAP	= {
		'base64'	: exampleBase64,
		'build'		: exampleBuild,
		'client'	: exampleClient,
		'date'		: exampleDate,
		'encode'	: exampleEncode,
		'nbClient'	: exampleNbClient,
		'server'	: exampleServer
	}

	xmlrpc.setLogLevel(LOGLEVEL)
	if len(sys.argv) <> 2:
		sys.stderr.write('Exactly one example must be specified\n')
		usage()
	example = sys.argv[1]
	if not TESTMAP.has_key(example):
		sys.stderr.write('Unknown example: %s\n' % example)
		usage()

	TESTMAP[example]()


def usage():
	global TESTMAP

	validKeys = map(lambda s: ('[%s]' % s), TESTMAP.keys())
	validKeys = string.join(validKeys)
	sys.stderr.write('usage: examples.py %s\n' % validKeys)

	sys.exit(1)


def exampleClient():
	r = 'Hello, World!'
	c = xmlrpc.client()
	c.connect('localhost', PORT, TIMEOUT)
	print c.execute('echo', ['Hello, World!'])
	print c.execute('echo', ['Hello, World!'], {}, TIMEOUT)


def exampleServer():
	s = xmlrpc.server()
	s.addMethods({'echo' : echoMethod, 'exit' : exitMethod})
	s.bindAndListen(PORT)
	while 1:
		try:
			s.work()	# you could set a timeout if desired
		except:
			traceback.print_exc()


def echoMethod(servp, srcp, method, params):
	return params


def exitMethod(servp, srcp, method, params):
	servp.exit()
	return 'okay'


def errHandler(*l):
	print 'in err handler', `l`
	return


def exampleNbClient():
	xmlrpc.setLogLevel(0)
	c = xmlrpc.client()
	c.connect('localhost', PORT)
	p = ["hello", "world"]
	r = c.nbExecute('echo', p, {}, callback, 'Extra Args')
	c.work(10)


def callback(client, response, extArgs=None):
	print 'callback got response', `response`
	print 'result is', xmlrpc.parseResponse(response)


def exampleBuild():
	request = xmlrpc.buildRequest('someMethod', ['Hello, World!'], {})
	print 'request is: ', `request`
	print
	print 'parsed request is',  xmlrpc.parseRequest(request)
	print

	response = xmlrpc.buildResponse(['Hello, World!'], {})
	print 'response is: ', `response`
	print
	print 'parsed response is', xmlrpc.parseResponse(response)
	print


def exampleEncode():
	r = ['hum', 3242, 'de']
	print 'object is', r
	print 'encoded is:\n', xmlrpc.encode(r)
	print 'decoded is:', xmlrpc.decode(xmlrpc.encode(r))


def exampleDate():
	d = xmlrpc.dateTime(1999, 6, 12, 4, 32, 34)
	print 'date is', d
	print 'encoded date is', xmlrpc.encode(d)
	print 'decoded date is', xmlrpc.decode(xmlrpc.encode(d))


def exampleBase64():
	b = xmlrpc.base64('Hello, world!')
	print 'base64 is ', b
	print 'encoded base64 is ', xmlrpc.encode(b)
	print 'decoded base64 is ', xmlrpc.decode(xmlrpc.encode(b))


main()
